﻿# ##########################################################################
# Programme Python simulant la propagation dans l'espace de deux ondes
# progressives périodiques de caractéristiques différentes
# Possibilité de mettre en pause l'animation en cliquant sur la fenêtre
#
# Les élèves ont juste à modifier les grandeurs physiques T2, v2 pour comprendre
# leur influence sur la propagation de l'onde
# ##########################################################################
# O. CHAUMETTE - Lycée JP SARTRE - 69500 BRON - olivier.chaumette@ac-lyon.fr
# ##########################################################################
# version 1 - décembre 2019

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation as animation

# ############################
# 1. FONCTIONS
# ############################
# fonction qui est applelé à chaque image de l'animation (frame)
def Trace_Chaque_Frame(i):
    # calcul de la date pour la frame i en cours:
    t = i * dt
    # calcul de y pour chaque onde avec l'équation d'onde:
    y1 = A1*np.cos(2*np.pi*(x/Lambda1) - 2*np.pi*(t/T1))
    y2 = A2*np.cos(2*np.pi*(x/Lambda2) - 2*np.pi*(t/T2))
    # on ajoute (x,y1) et (x,y2) à chaque liste oonde1 et onde2
    onde1.set_data(x, y1)
    onde2.set_data(x, y2)
    # on ajoute les données x,y pour tracer les deux lignes en pointillés
    # ces lignes sont verticales en x= 0,5  et x=1 (1/2 lgr d'onde et lgr d'onde)
    trait1.set_data([0.5,0.5],[-1.5*A1,1.5*A1])
    trait2.set_data([1,1],[-1.5*A1,1.5*A1])
    # on retourne les 4 listes pour le tracé
    return onde1,onde2,trait1,trait2

# fonction mettant en pause ou non l'animation si on clique sur la fenêtre matplotlib
def Clic(event):
    global animation_en_pause
    if not animation_en_pause:
        Figure_anime.event_source.stop()
        animation_en_pause = True
    else:
        Figure_anime.event_source.start()
        animation_en_pause = False

# ############################
# 2. initialisation des variables
# ############################
# ONDE n° 1 de référence
# période en secondes:
T1=0.5
# célérité en m/s:
v1=2
# amplitude en mètres
A1=2

""" ---------------------------------------------
    ------ PARTIE MODIFIABLE PAR LES ÉLÈVES -----
------------------------------------------------- """
# ONDE n° 2
# période en secondes:
T2=0.5
# célérité en m/s:
v2=2
# amplitude en mètres
A2=2
""" FIN PARTIE MODIFIABLE PAR LES ÉLÈVES --------- """

# Calcul des longeurs d'onde:
Lambda1=v1*T1
Lambda2=v2*T2
# intervalle de temps entre deux images de l'animation
dt = 0.01
# tableaux numpy des positions sur l'axe des abscisses:
xmin = 0
xmax = 4
nb_valeurs_x = 49
x = np.linspace(xmin, xmax, nb_valeurs_x)

# variable booléenne permettant de savoir si l'animation est en pause ou non
animation_en_pause = False

# ############################
# 3. Préparation de la figure
# ############################
# initialisation de la figure:
Figure = plt.figure("Propagation de 2 ondes périodiques progressives")
# Création d'un évènement "Clic de la souris" sur la fenêtre Matplolib (si clic, appel de la fonction "Clic")
Figure.canvas.mpl_connect('button_press_event', Clic)

# ONDE 1 ############################################
# séparation en 2 sous figures (2 lignes, 1 colonne):
# 1ère ligne sélectionnée pour tracer l'onde 1:
plt.subplot(2,1,1)
# définition du style de l'onde 1:
onde1, = plt.plot([],[],"ro--")
# définition du style des 2 lignes verticales qui permetent de mieux repérer 2 points:
trait1, = plt.plot([],[],"k--")
trait2, = plt.plot([],[],"k--")
# limites des axes pour bien voir la figure. 1.5*A évite que les courbes soient au mxi de la fenêtre:
plt.xlim(xmin, xmax)
plt.ylim(-1.5*A1,1.5*A1)
# textes sur les axes et titre de la figure
plt.xlabel("abscisse x en m")
plt.ylabel("ordonnée y des points en m")
plt.title("Onde n°1 de référence")
# possibilité d'afficher les informations:
#plt.title("T= "+str(T1)+" s   v="+str(v1)+" m/s     "+chr(955)+" = "+str(Lambda1)+" m")

# ONDE 2 ############################################
# séparation en 2 sous figures (2 lignes, 1 colonne):
# 2nde ligne sélectionnée pour tracer l'onde 2
plt.subplot(2,1,2)
# définition du style de l'onde 2
onde2, = plt.plot([],[],"bo--")
# axes etc... pour l'onde 2
plt.xlim(xmin, xmax)
plt.ylim(-1.5*A2,1.5*A2)
plt.xlabel("abscisse x en m")
plt.ylabel("ordonnée y des points en m")
plt.title("Onde n°2")
# possibilité d'afficher les informations:
# plt.title("T= "+str(T2)+" s   v="+str(v2)+" m/s     "+chr(955)+" = "+str(Lambda2)+" m")

# réglage automatiques des marges entre les figures:
plt.tight_layout()

# ############################
# 4. Animation de la figure
# ############################
Figure_anime = animation.FuncAnimation(Figure, Trace_Chaque_Frame, frames=nb_valeurs_x, blit=True, interval=10, repeat=True)

plt.show()
